@@ -29,9 +29,12 @@ from .cv.models.diffusion import ldm as _ldm
 from .cv.models.diffusion import ldm_vq as _ldm_vq
 from .cv.models.diffusion import ldm_sd as _ldm_sd
 from .cv.models.diffusion import ldm_sr as _ldm_sr
+from .cv.models.diffusion import ldm_sd_anime as _ldm_sd_anime
 from .cv.models.diffusion import ldm_semantic as _ldm_semantic
 from .cv.models.diffusion import ldm_celeba_hq as _ldm_celeba_hq
 from .cv.models.diffusion import ldm_inpainting as _ldm_inpainting
+from .cv.models.translator import esr as _esr
+from .cv.models.translator import esr_anime as _esr_anime
 from ..data import MLData
 from ..data import IMLData
 from ..data import CVDataModule
@@ -1335,6 +1338,10 @@ def ldm_sd(pretrained: bool = True) -> DLPipeline:
     return _ldm_sd(pretrained)
 
 
+def ldm_sd_anime(pretrained: bool = True) -> DLPipeline:
+    return _ldm_sd_anime(pretrained)
+
+
 def ldm_celeba_hq(pretrained: bool = True) -> DLPipeline:
     return _ldm_celeba_hq(pretrained)
 
@@ -1351,6 +1358,17 @@ def ldm_semantic(pretrained: bool = True) -> DLPipeline:
     return _ldm_semantic(pretrained)
 
 
+# sr
+
+
+def esr(pretrained: bool = True) -> DLPipeline:
+    return _esr(pretrained)
+
+
+def esr_anime(pretrained: bool = True) -> DLPipeline:
+    return _esr_anime(pretrained)
+
+
 # nlp
 
 
