@@ -1,4 +1,5 @@
 import torch
+import random
 
 import numpy as np
 import torch.nn as nn
@@ -14,92 +15,68 @@ from typing import Tuple
 from typing import Union
 from typing import Callable
 from typing import Optional
-from typing import NamedTuple
 from cftool.misc import safe_execute
 from cftool.misc import shallow_copy_dict
 from cftool.array import save_images
 from cftool.types import tensor_dict_type
-from torch.cuda.amp.autocast_mode import autocast
 
+from .common import read_image
+from .common import restrict_wh
+from .common import get_suitable_size
+from .common import APIMixin
 from ....zoo import DLZoo
 from ....data import predict_tensor_data
 from ....data import TensorInferenceData
 from ....pipeline import DLPipeline
 from ....constants import INPUT_KEY
+from ....misc.toolkit import slerp
+from ....misc.toolkit import new_seed
 from ....misc.toolkit import eval_context
+from ....misc.toolkit import seed_everything
+from ....modules.blocks import Conv2d
 from ....models.cv.diffusion import LDM
 from ....models.cv.diffusion import DDPM
 from ....models.cv.diffusion import ISampler
-from ....models.cv.diffusion import DDIMSampler
-from ....models.cv.diffusion import PLMSSampler
 from ....models.cv.ae.common import IAutoEncoder
 from ....models.cv.diffusion.utils import get_timesteps
-
-try:
-    from cfcv.misc.toolkit import to_rgb
-except:
-    to_rgb = None
+from ....models.cv.diffusion.samplers.ddim import DDIMMixin
+from ....models.cv.diffusion.samplers.k_samplers import KSamplerMixin
 
 
-def is_ddim(sampler: ISampler) -> bool:
-    return isinstance(sampler, (DDIMSampler, PLMSSampler))
+class switch_sampler_context:
+    def __init__(self, api: "DiffusionAPI", sampler: str):
+        self.api = api
+        self.m_sampler = api.sampler.__identifier__
+        self.target_sampler = sampler
+        self.switched = False
+
+    def __enter__(self) -> None:
+        if self.m_sampler != self.target_sampler:
+            self.switched = True
+            self.api.switch_sampler(self.target_sampler)
+
+    def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
+        if self.switched:
+            self.api.switch_sampler(self.m_sampler)
 
 
-def get_suitable_size(n: int, anchor: int = 64) -> int:
-    mod = n % anchor
-    return n - mod + int(mod > 0.5 * anchor) * anchor
-
-
-class ReadImageResponse(NamedTuple):
-    image: np.ndarray
-    original_size: Tuple[int, int]
-
-
-def read_image(
-    path: str,
-    max_wh: int,
-    *,
-    to_gray: bool = False,
-    resample: Any = Image.LANCZOS,
-    normalize: bool = True,
-) -> ReadImageResponse:
-    if to_rgb is None:
-        raise ValueError("`carefree-cv` is needed for `DiffusionAPI`")
-    image = Image.open(path)
-    image = image.convert("L") if to_gray else to_rgb(image)
-    original_w, original_h = image.size
-    max_original_wh = max(original_w, original_h)
-    if max_original_wh <= max_wh:
-        w, h = original_w, original_h
-    else:
-        wh_ratio = original_w / original_h
-        if wh_ratio >= 1:
-            w = max_wh
-            h = round(w / wh_ratio)
-        else:
-            h = max_wh
-            w = round(h * wh_ratio)
-    w, h = map(get_suitable_size, (w, h))
-    image = image.resize((w, h), resample=resample)
-    image = np.array(image)
-    if normalize:
-        image = image.astype(np.float32) / 255.0
-    if to_gray:
-        image = image[None, None]
-    else:
-        image = image[None].transpose(0, 3, 1, 2)
-    return ReadImageResponse(image, (original_w, original_h))
-
-
-class DiffusionAPI:
+class DiffusionAPI(APIMixin):
     m: DDPM
     sampler: ISampler
     cond_model: Optional[nn.Module]
     first_stage: Optional[IAutoEncoder]
+    latest_seed: int
+    latest_variation_seed: Optional[int]
 
-    def __init__(self, m: DDPM, *, use_amp: bool = False):
-        self.m = m
-        self.use_amp = use_amp
+    def __init__(
+        self,
+        m: DDPM,
+        device: torch.device,
+        *,
+        use_amp: bool = False,
+        use_half: bool = False,
+    ):
+        super().__init__(m, device, use_amp=use_amp, use_half=use_half)
         self.sampler = m.sampler
         self.cond_type = m.condition_type
         # extracted the condition model so we can pre-calculate the conditions
@@ -108,20 +85,16 @@ class DiffusionAPI:
             self.cond_model.eval()
         m.condition_model = nn.Identity()
         # pre-calculate unconditional_cond if needed
-        if is_ddim(m.sampler):
-            if self.cond_model is not None and m.sampler.unconditional_cond is not None:
-                uncond = self.get_cond(m.sampler.unconditional_cond)
-                m.sampler.unconditional_cond = uncond.to(m.device)
+        unconditional_cond = getattr(m.sampler, "unconditional_cond", None)
+        if self.cond_model is not None and unconditional_cond is not None:
+            uncond = self.get_cond(m.sampler.unconditional_cond)
+            m.sampler.unconditional_cond = uncond.to(self.device)
         # extract first stage
         if not isinstance(m, LDM):
             self.first_stage = None
         else:
             self.first_stage = m.first_stage.core
 
-    @property
-    def amp_context(self) -> autocast:
-        return autocast(enabled=self.use_amp)
-
     def get_cond(self, cond: Any) -> Tensor:
         if self.cond_model is None:
             msg = "should not call `get_cond` when `cond_model` is not available"
@@ -136,19 +109,48 @@ class DiffusionAPI:
         sampler_config: Optional[Dict[str, Any]] = None,
     ) -> None:
         sampler_ins = self.m.make_sampler(sampler, sampler_config)
-        if is_ddim(self.m.sampler) and is_ddim(sampler_ins):
-            sampler_ins.unconditional_cond = self.m.sampler.unconditional_cond  # type: ignore
+        current_unconditional_cond = getattr(self.m.sampler, "unconditional_cond", None)
+        if current_unconditional_cond is not None:
+            if hasattr(sampler_ins, "unconditional_cond"):
+                sampler_ins.unconditional_cond = current_unconditional_cond
+        current_guidance = getattr(self.m.sampler, "unconditional_guidance_scale", None)
+        if current_guidance is not None:
+            if hasattr(sampler_ins, "unconditional_guidance_scale"):
+                sampler_ins.unconditional_guidance_scale = current_guidance
+        self.sampler = self.m.sampler = sampler_ins
+
+    def switch_circular(self, enable: bool) -> None:
+        def _inject(m: nn.Module) -> None:
+            for child in m.children():
+                _inject(child)
+            modules.append(m)
+
+        padding_mode = "circular" if enable else "zeros"
+        modules: List[nn.Module] = []
+        _inject(self.m)
+        for module in modules:
+            if isinstance(module, nn.Conv2d):
+                module.padding_mode = padding_mode
+            elif isinstance(module, Conv2d):
+                module.padding = padding_mode
 
     def sample(
         self,
         num_samples: int,
         export_path: Optional[str] = None,
         *,
+        seed: Optional[int] = None,
+        # each variation contains (seed, weight)
+        variations: Optional[List[Tuple[int, float]]] = None,
+        variation_seed: Optional[int] = None,
+        variation_strength: Optional[float] = None,
         z: Optional[Tensor] = None,
         z_ref: Optional[Tensor] = None,
         z_ref_mask: Optional[Tensor] = None,
+        z_ref_noise: Optional[Tensor] = None,
         size: Optional[Tuple[int, int]] = None,
         original_size: Optional[Tuple[int, int]] = None,
+        alpha: Optional[np.ndarray] = None,
         cond: Optional[Any] = None,
         num_steps: Optional[int] = None,
         clip_output: bool = True,
@@ -166,6 +168,13 @@ class DiffusionAPI:
                 f"`num_samples` ({num_samples}) should be identical with "
                 f"the number of `cond` ({len(cond)})"
             )
+        if alpha is not None and original_size is not None:
+            alpha_h, alpha_w = alpha.shape[-2:]
+            if alpha_w != original_size[0] or alpha_h != original_size[1]:
+                raise ValueError(
+                    f"shape of the provided `alpha` ({alpha_w}, {alpha_h}) should be "
+                    f"identical with the provided `original_size` {original_size}"
+                )
         unconditional = cond is None
         if unconditional:
             cond = [0] * num_samples
@@ -188,17 +197,34 @@ class DiffusionAPI:
             with self.amp_context:
                 for batch in iterator:
                     i_kw = shallow_copy_dict(kw)
-                    i_cond = batch[INPUT_KEY].to(self.m.device)
+                    i_cond = batch[INPUT_KEY].to(self.device)
+                    repeat = lambda t: t.repeat_interleave(len(i_cond), dim=0)
                     if z is not None:
-                        i_z = z.repeat_interleave(len(i_cond), dim=0)
+                        i_z = repeat(z)
                     else:
                         i_z_shape = len(i_cond), self.m.in_channels, *size[::-1]
-                        i_z = torch.randn(i_z_shape, device=self.m.device)
+                        i_z, _ = self._set_seed_and_variations(
+                            seed,
+                            lambda: torch.randn(i_z_shape, device=self.device),
+                            lambda noise: noise,
+                            variations,
+                            variation_seed,
+                            variation_strength,
+                        )
+                    if z_ref is not None and z_ref_mask is not None:
+                        if z_ref_noise is not None:
+                            i_kw["ref"] = repeat(z_ref)
+                            i_kw["ref_mask"] = repeat(z_ref_mask)
+                            i_kw["ref_noise"] = repeat(z_ref_noise)
                     if unconditional:
                         i_cond = None
-                    if z_ref is not None and z_ref_mask is not None:
-                        i_kw["ref"] = z_ref
-                        i_kw["ref_mask"] = z_ref_mask
+                    if self.use_half:
+                        i_z = i_z.half()
+                        if i_cond is not None:
+                            i_cond = i_cond.half()
+                        for k, v in i_kw.items():
+                            if isinstance(v, torch.Tensor) and v.is_floating_point():
+                                i_kw[k] = v.half()
                     i_sampled = self.m.decode(i_z, cond=i_cond, **i_kw)
                     sampled.append(i_sampled.cpu().float())
         concat = torch.cat(sampled, dim=0)
@@ -214,8 +240,19 @@ class DiffusionAPI:
                     mode="bicubic",
                     antialias=True,
                 )
+        if alpha is not None:
+            alpha = torch.from_numpy(2.0 * alpha - 1.0)
+            if original_size is None:
+                with torch.no_grad():
+                    alpha = F.interpolate(
+                        alpha,
+                        concat.shape[-2:],
+                        mode="nearest",
+                    )
+            concat = torch.cat([concat, alpha], dim=1)
         if export_path is not None:
             save_images(concat, export_path)
+        torch.cuda.empty_cache()
         return concat
 
     def txt2img(
@@ -223,6 +260,8 @@ class DiffusionAPI:
         txt: Union[str, List[str]],
         export_path: Optional[str] = None,
         *,
+        anchor: int = 64,
+        max_wh: int = 512,
         num_samples: Optional[int] = None,
         size: Optional[Tuple[int, int]] = None,
         num_steps: Optional[int] = None,
@@ -245,11 +284,12 @@ class DiffusionAPI:
         if size is None:
             new_size = None
         else:
-            new_size = tuple(map(get_suitable_size, size))
+            new_size = restrict_wh(*size, max_wh)
+            new_size = tuple(map(get_suitable_size, new_size, (anchor, anchor)))  # type: ignore
         return self.sample(
             num_samples,
             export_path,
-            size=new_size,  # type: ignore
+            size=new_size,
             original_size=size,
             cond=txt,
             num_steps=num_steps,
@@ -260,26 +300,69 @@ class DiffusionAPI:
             **kwargs,
         )
 
-    def img2img(
+    def outpainting(
         self,
-        img_path: str,
+        txt: str,
+        image: Union[str, Image.Image],
         export_path: Optional[str] = None,
         *,
+        anchor: int = 32,
         max_wh: int = 512,
         fidelity: float = 0.2,
+        padding_mode: str = "cv2_telea",
+        num_steps: Optional[int] = None,
+        clip_output: bool = True,
+        verbose: bool = True,
+        **kwargs: Any,
+    ) -> Tensor:
+        with switch_sampler_context(self, "plms"):
+            res = read_image(image, max_wh, anchor=anchor, padding_mode=padding_mode)
+            if res.alpha is None:
+                raise ValueError("`image` should contain alpha channel in outpainting")
+            z = z_ref = self._get_z(res.image)
+            z_ref_mask = F.interpolate(
+                torch.from_numpy(res.alpha).to(z_ref),
+                z_ref.shape[-2:],
+                mode="bicubic",
+            )
+            return self._img2img(
+                z,
+                export_path,
+                z_ref=z_ref,
+                z_ref_mask=z_ref_mask,
+                fidelity=fidelity,
+                original_size=res.original_size,
+                alpha=None,
+                cond=[txt],
+                num_steps=num_steps,
+                clip_output=clip_output,
+                verbose=verbose,
+                **kwargs,
+            )
+
+    def img2img(
+        self,
+        image: Union[str, Image.Image],
+        export_path: Optional[str] = None,
+        *,
+        anchor: int = 32,
+        max_wh: int = 512,
+        fidelity: float = 0.2,
+        alpha: Optional[np.ndarray] = None,
         cond: Optional[Any] = None,
         num_steps: Optional[int] = None,
         clip_output: bool = True,
         verbose: bool = True,
         **kwargs: Any,
     ) -> Tensor:
-        res = read_image(img_path, max_wh)
+        res = read_image(image, max_wh, anchor=anchor)
         z = self._get_z(res.image)
         return self._img2img(
             z,
             export_path,
             fidelity=fidelity,
             original_size=res.original_size,
+            alpha=res.alpha if alpha is None else alpha,
             cond=cond,
             num_steps=num_steps,
             clip_output=clip_output,
@@ -289,11 +372,13 @@ class DiffusionAPI:
 
     def inpainting(
         self,
-        img_path: str,
-        mask_path: str,
+        image: Union[str, Image.Image],
+        mask: Union[str, Image.Image],
         export_path: Optional[str] = None,
         *,
+        anchor: int = 32,
         max_wh: int = 512,
+        alpha: Optional[np.ndarray] = None,
         refine_fidelity: Optional[float] = None,
         num_steps: Optional[int] = None,
         clip_output: bool = True,
@@ -307,16 +392,17 @@ class DiffusionAPI:
             return 2.0 * final - 1.0
 
         # handle mask stuffs
-        image_res = read_image(img_path, max_wh)
-        mask = read_image(mask_path, max_wh, to_gray=True).image
+        image_res = read_image(image, max_wh, anchor=anchor)
+        mask = read_image(mask, max_wh, anchor=anchor, to_mask=True).image
         bool_mask = mask >= 0.5
-        remained_mask = (~bool_mask).astype(np.float32)
+        remained_mask = (~bool_mask).astype(np.float16 if self.use_half else np.float32)
         remained_image = remained_mask * image_res.image
         # construct condition tensor
         remained_cond = self._get_z(remained_image)
         latent_shape = remained_cond.shape[-2:]
         mask_cond = torch.where(torch.from_numpy(bool_mask), 1.0, -1.0)
-        mask_cond = mask_cond.to(torch.float32).to(self.m.device)
+        mask_cond = mask_cond.to(torch.float16 if self.use_half else torch.float32)
+        mask_cond = mask_cond.to(self.device)
         mask_cond = F.interpolate(mask_cond, size=latent_shape)
         cond = torch.cat([remained_cond, mask_cond], dim=1)
         # refine with img2img
@@ -327,6 +413,7 @@ class DiffusionAPI:
                 export_path,
                 fidelity=refine_fidelity,
                 original_size=image_res.original_size,
+                alpha=image_res.alpha if alpha is None else alpha,
                 cond=cond,
                 num_steps=num_steps,
                 clip_output=clip_output,
@@ -340,6 +427,7 @@ class DiffusionAPI:
             export_path,
             z=z,
             original_size=image_res.original_size,
+            alpha=image_res.alpha if alpha is None else alpha,
             cond=cond,
             num_steps=num_steps,
             clip_output=clip_output,
@@ -350,10 +438,12 @@ class DiffusionAPI:
 
     def sr(
         self,
-        img_path: str,
+        image: Union[str, Image.Image],
         export_path: Optional[str] = None,
         *,
+        anchor: int = 8,
         max_wh: int = 512,
+        alpha: Optional[np.ndarray] = None,
         num_steps: Optional[int] = None,
         clip_output: bool = True,
         verbose: bool = True,
@@ -362,16 +452,19 @@ class DiffusionAPI:
         if not isinstance(self.m, LDM):
             raise ValueError("`sr` is now only available for `LDM` models")
         factor = 2 ** (len(self.m.first_stage.core.channel_multipliers) - 1)
-        res = read_image(img_path, round(max_wh / factor))
-        cond = torch.from_numpy(2.0 * res.image - 1.0).to(self.m.device)
-        w, h = res.original_size
-        sr_size = w * factor, h * factor
+        res = read_image(image, round(max_wh / factor), anchor=anchor)
+        wh_ratio = res.original_size[0] / res.original_size[1]
+        zh, zw = res.image.shape[-2:]
+        sr_size = (zw, zw / wh_ratio) if zw > zh else (zh * wh_ratio, zh)
+        sr_size = tuple(map(lambda n: round(factor * n), sr_size))  # type: ignore
+        cond = torch.from_numpy(2.0 * res.image - 1.0).to(self.device)
         z = torch.randn_like(cond)
         return self.sample(
             1,
             export_path,
             z=z,
             original_size=sr_size,
+            alpha=res.alpha if alpha is None else alpha,
             cond=cond,
             num_steps=num_steps,
             clip_output=clip_output,
@@ -381,10 +474,12 @@ class DiffusionAPI:
 
     def semantic2img(
         self,
-        semantic_path: str,
+        semantic: Union[str, Image.Image],
         export_path: Optional[str] = None,
         *,
+        anchor: int = 16,
         max_wh: int = 512,
+        alpha: Optional[np.ndarray] = None,
         num_steps: Optional[int] = None,
         clip_output: bool = True,
         verbose: bool = True,
@@ -400,14 +495,16 @@ class DiffusionAPI:
         if factor is None:
             raise ValueError(err_fmt.format("cond_model.factor"))
         res = read_image(
-            semantic_path,
+            semantic,
             max_wh,
+            anchor=anchor,
             to_gray=True,
             resample=Image.NEAREST,
             normalize=False,
         )
-        cond = torch.from_numpy(res.image).to(torch.long).to(self.m.device)
-        cond = F.one_hot(cond, num_classes=in_channels)[0].float()
+        cond = torch.from_numpy(res.image).to(torch.long).to(self.device)
+        cond = F.one_hot(cond, num_classes=in_channels)[0]
+        cond = cond.half() if self.use_half else cond.float()
         cond = cond.permute(0, 3, 1, 2).contiguous()
         cond = self.get_cond(cond)
         z = torch.randn_like(cond)
@@ -416,6 +513,7 @@ class DiffusionAPI:
             export_path,
             z=z,
             original_size=res.original_size,
+            alpha=res.alpha if alpha is None else alpha,
             cond=cond,
             num_steps=num_steps,
             clip_output=clip_output,
@@ -423,26 +521,26 @@ class DiffusionAPI:
             **kwargs,
         )
 
-    @classmethod
-    def from_pipeline(
-        cls,
-        m: DLPipeline,
-        device: Optional[str] = None,
-        *,
-        use_amp: bool = False,
-    ) -> "DiffusionAPI":
-        if device is not None:
-            m.model.to(device)
-        return cls(m.model.core, use_amp=use_amp)
-
     @classmethod
     def from_sd(
         cls,
         device: Optional[str] = None,
         *,
         use_amp: bool = False,
+        use_half: bool = False,
     ) -> "DiffusionAPI":
-        return cls.from_pipeline(ldm_sd(), device, use_amp=use_amp)
+        return cls.from_pipeline(ldm_sd(), device, use_amp=use_amp, use_half=use_half)
+
+    @classmethod
+    def from_sd_anime(
+        cls,
+        device: Optional[str] = None,
+        *,
+        use_amp: bool = False,
+        use_half: bool = False,
+    ) -> "DiffusionAPI":
+        m = ldm_sd_anime()
+        return cls.from_pipeline(m, device, use_amp=use_amp, use_half=use_half)
 
     @classmethod
     def from_celeba_hq(
@@ -450,8 +548,10 @@ class DiffusionAPI:
         device: Optional[str] = None,
         *,
         use_amp: bool = False,
+        use_half: bool = False,
     ) -> "DiffusionAPI":
-        return cls.from_pipeline(ldm_celeba_hq(), device, use_amp=use_amp)
+        m = ldm_celeba_hq()
+        return cls.from_pipeline(m, device, use_amp=use_amp, use_half=use_half)
 
     @classmethod
     def from_inpainting(
@@ -459,8 +559,10 @@ class DiffusionAPI:
         device: Optional[str] = None,
         *,
         use_amp: bool = False,
+        use_half: bool = False,
     ) -> "DiffusionAPI":
-        return cls.from_pipeline(ldm_inpainting(), device, use_amp=use_amp)
+        m = ldm_inpainting()
+        return cls.from_pipeline(m, device, use_amp=use_amp, use_half=use_half)
 
     @classmethod
     def from_sr(
@@ -468,8 +570,9 @@ class DiffusionAPI:
         device: Optional[str] = None,
         *,
         use_amp: bool = False,
+        use_half: bool = False,
     ) -> "DiffusionAPI":
-        return cls.from_pipeline(ldm_sr(), device, use_amp=use_amp)
+        return cls.from_pipeline(ldm_sr(), device, use_amp=use_amp, use_half=use_half)
 
     @classmethod
     def from_semantic(
@@ -477,22 +580,66 @@ class DiffusionAPI:
         device: Optional[str] = None,
         *,
         use_amp: bool = False,
+        use_half: bool = False,
     ) -> "DiffusionAPI":
-        return cls.from_pipeline(ldm_semantic(), device, use_amp=use_amp)
+        m = ldm_semantic()
+        return cls.from_pipeline(m, device, use_amp=use_amp, use_half=use_half)
 
     def _get_z(self, img: np.ndarray) -> Tensor:
         img = 2.0 * img - 1.0
-        z = torch.from_numpy(img).to(self.m.device)
-        z = self.m._preprocess(z)
+        z = torch.from_numpy(img)
+        if self.use_half:
+            z = z.half()
+        z = z.to(self.device)
+        z = self.m._preprocess(z, deterministic=True)
         return z
 
+    def _set_seed_and_variations(
+        self,
+        seed: Optional[int],
+        get_noise: Callable[[], Tensor],
+        get_new_z: Callable[[Tensor], Tensor],
+        variations: Optional[List[Tuple[int, float]]],
+        variation_seed: Optional[int],
+        variation_strength: Optional[float],
+    ) -> Tuple[Tensor, Tensor]:
+        if seed is None:
+            seed = new_seed()
+        seed = seed_everything(seed)
+        self.latest_seed = seed
+        z_noise = get_noise()
+        self.latest_variation_seed = None
+        if variations is not None:
+            for v_seed, v_weight in variations:
+                seed_everything(v_seed)
+                v_noise = get_noise()
+                z_noise = slerp(v_noise, z_noise, v_weight)
+        if variation_strength is not None:
+            random.seed()
+            if variation_seed is None:
+                variation_seed = new_seed()
+            variation_seed = seed_everything(variation_seed)
+            self.latest_variation_seed = variation_seed
+            variation_noise = get_noise()
+            z_noise = slerp(variation_noise, z_noise, variation_strength)
+        z = get_new_z(z_noise)
+        return z, z_noise
+
     def _img2img(
         self,
         z: Tensor,
         export_path: Optional[str] = None,
         *,
+        seed: Optional[int] = None,
+        # each variation contains (seed, weight)
+        variations: Optional[List[Tuple[int, float]]] = None,
+        variation_seed: Optional[int] = None,
+        variation_strength: Optional[float] = None,
+        z_ref: Optional[Tensor] = None,
+        z_ref_mask: Optional[Tensor] = None,
         fidelity: float = 0.2,
         original_size: Optional[Tuple[int, int]] = None,
+        alpha: Optional[np.ndarray] = None,
         cond: Optional[Any] = None,
         num_steps: Optional[int] = None,
         clip_output: bool = True,
@@ -503,19 +650,29 @@ class DiffusionAPI:
         if num_steps is None:
             num_steps = self.sampler.default_steps
         t = round((1.0 - fidelity) * num_steps)
-        ts = get_timesteps(t, 1, z.device)
-        if isinstance(self.sampler, (DDIMSampler, PLMSSampler)):
+        ts = get_timesteps(min(t, num_steps - 1), 1, z.device)
+        if isinstance(self.sampler, (DDIMMixin, KSamplerMixin)):
             kw = shallow_copy_dict(self.sampler.sample_kwargs)
             kw["total_step"] = num_steps
             safe_execute(self.sampler._reset_buffers, kw)
-        z = self.sampler.q_sample(z, ts)
+        z, noise = self._set_seed_and_variations(
+            seed,
+            lambda: torch.randn_like(z),
+            lambda noise_: self.sampler.q_sample(z, ts, noise_),
+            variations,
+            variation_seed,
+            variation_strength,
+        )
         kwargs["start_step"] = num_steps - t
-        # sampling
         return self.sample(
             1,
             export_path,
             z=z,
+            z_ref=z_ref,
+            z_ref_mask=z_ref_mask,
+            z_ref_noise=None if z_ref is None else noise,
             original_size=original_size,
+            alpha=alpha,
             cond=cond,
             num_steps=num_steps,
             clip_output=clip_output,
@@ -582,8 +739,12 @@ def ldm_vq(
     )
 
 
-def ldm_sd(pretrained: bool = True) -> DLPipeline:
-    return _ldm("diffusion/ldm.sd", 64, 4, 4, pretrained=pretrained)
+def ldm_sd(pretrained: bool = True, **kwargs: Any) -> DLPipeline:
+    return _ldm("diffusion/ldm.sd", 64, 4, 4, pretrained=pretrained, **kwargs)
+
+
+def ldm_sd_anime(pretrained: bool = True) -> DLPipeline:
+    return ldm_sd(pretrained, download_name="ldm_sd_anime")
 
 
 def ldm_celeba_hq(pretrained: bool = True) -> DLPipeline:
