@@ -4,21 +4,51 @@ import torch
 import numpy as np
 import torch.nn.functional as F
 
+from abc import abstractmethod
 from abc import ABCMeta
 from torch import Tensor
 from typing import Any
 from typing import Dict
 from typing import Tuple
 from typing import Optional
+from typing import Protocol
 
 from .protocol import ISampler
 from .protocol import IDiffusion
-from ..utils import q_sample
+from .protocol import DDPMQSampler
+from .protocol import UncondSamplerMixin
 from ..utils import get_timesteps
 from ...ae.vq import AutoEncoderVQModel
 
 
-class DDIMMixin(ISampler, metaclass=ABCMeta):
+class IGetEPSPred(Protocol):
+    def __call__(self, image: Tensor, ts: Tensor) -> Tensor:
+        pass
+
+
+class IGetDenoised(Protocol):
+    def __call__(self, eps: Tensor) -> Tensor:
+        pass
+
+
+class DDIMQSampler(DDPMQSampler):
+    def reset_buffers(self, discretize: str, total_step: int) -> None:  # type: ignore
+        if discretize == "uniform":
+            span = self.model.t // total_step
+            q_timesteps = np.array(list(range(0, self.model.t, span)))
+        elif discretize == "quad":
+            end = math.sqrt(self.model.t * 0.8)
+            q_timesteps = (np.linspace(0, end, total_step) ** 2).astype(int)
+        else:
+            raise ValueError(f"unrecognized discretize method '{discretize}' occurred")
+        q_timesteps += 1
+        alphas = self.model.alphas_cumprod
+        self.alphas = alphas[q_timesteps]
+        self.timesteps = q_timesteps
+        super().reset_buffers(torch.sqrt(self.alphas), torch.sqrt(1.0 - self.alphas))
+
+
+class DDIMMixin(ISampler, UncondSamplerMixin, metaclass=ABCMeta):
     def __init__(
         self,
         model: IDiffusion,
@@ -44,6 +74,35 @@ class DDIMMixin(ISampler, metaclass=ABCMeta):
         self.quantize_denoised = quantize_denoised
         self.default_steps = default_steps
 
+    # abstract
+
+    @abstractmethod
+    def sample_step_core(
+        self,
+        image: Tensor,
+        cond: Optional[Tensor],
+        step: int,
+        total_step: int,
+        get_eps_pred: IGetEPSPred,
+        get_denoised: IGetDenoised,
+        *,
+        eta: float,
+        discretize: str,
+        unconditional_cond: Optional[Any],
+        unconditional_guidance_scale: float,
+        temperature: float,
+        noise_dropout: float,
+        quantize_denoised: bool,
+        **kwargs: Any,
+    ) -> Tensor:
+        pass
+
+    # inheritance
+
+    @property
+    def q_sampler(self) -> DDIMQSampler:
+        return self._q_sampler
+
     @property
     def sample_kwargs(self) -> Dict[str, Any]:
         return dict(
@@ -56,23 +115,55 @@ class DDIMMixin(ISampler, metaclass=ABCMeta):
             quantize_denoised=self.quantize_denoised,
         )
 
-    def q_sample(self, net: Tensor, timesteps: Tensor) -> Tensor:
-        return q_sample(
-            net,
-            timesteps,
-            torch.sqrt(self.ddim_alphas),
-            self.ddim_sqrt_one_minus_alphas,
+    def sample_step(
+        self,
+        image: Tensor,
+        cond: Optional[Tensor],
+        step: int,
+        total_step: int,
+        *,
+        eta: float = 0.0,
+        discretize: str = "uniform",
+        unconditional_cond: Optional[Any] = None,
+        unconditional_guidance_scale: float = 1.0,
+        temperature: float = 1.0,
+        noise_dropout: float = 1.0,
+        quantize_denoised: bool = False,
+        **kwargs: Any,
+    ) -> Tensor:
+        if step == 0:
+            self._reset_buffers(
+                eta,
+                discretize,
+                total_step,
+                unconditional_cond,
+                unconditional_guidance_scale,
+            )
+        self._register_temp_buffers(image, step, total_step)
+        return self.sample_step_core(
+            image,
+            cond,
+            step,
+            total_step,
+            lambda img, ts: self._uncond_denoise(img, ts, cond),
+            lambda eps: self._get_denoised_and_pred_x0(
+                eps,
+                image,
+                quantize_denoised,
+                temperature,
+                noise_dropout,
+            )[0],
+            eta=eta,
+            discretize=discretize,
+            unconditional_cond=unconditional_cond,
+            unconditional_guidance_scale=unconditional_guidance_scale,
+            temperature=temperature,
+            noise_dropout=noise_dropout,
+            quantize_denoised=quantize_denoised,
+            **kwargs,
         )
 
-    def _denoise(self, image: Tensor, ts: Tensor, cond: Optional[Tensor]) -> Tensor:
-        if cond is None or self.uncond is None:
-            return self.model.denoise(image, ts, cond)
-        uncond = self.uncond.repeat_interleave(cond.shape[0], dim=0)
-        cond2 = torch.cat([uncond, cond])
-        image2 = torch.cat([image, image])
-        ts2 = torch.cat([ts, ts])
-        eps_uncond, eps = self.model.denoise(image2, ts2, cond2).chunk(2)
-        return eps_uncond + self.uncond_guidance_scale * (eps - eps_uncond)
+    # internal
 
     def _register_temp_buffers(self, image: Tensor, step: int, total_step: int) -> None:
         b = image.shape[0]
@@ -82,7 +173,12 @@ class DDIMMixin(ISampler, metaclass=ABCMeta):
         t_next = self.ddim_timesteps[max(index - 1, 0)]
         self._ts = get_timesteps(t, b, device)
         self._ts_next = get_timesteps(t_next, b, device)
-        extract = lambda base: torch.full((b, 1, 1, 1), base[index], device=device)
+        extract = lambda base: torch.full(
+            (b, 1, 1, 1),
+            base[index],
+            dtype=image.dtype,
+            device=device,
+        )
         self._at = extract(self.ddim_alphas)
         self._a_prev_t = extract(self.ddim_alphas_prev)
         self._sigmas_t = extract(self.ddim_sigmas)
@@ -125,21 +221,13 @@ class DDIMMixin(ISampler, metaclass=ABCMeta):
         unconditional_cond: Optional[Any],
         unconditional_guidance_scale: float,
     ) -> None:
-        # discretize time steps
-        if discretize == "uniform":
-            span = self.model.t // total_step
-            ddim_timesteps = np.array(list(range(0, self.model.t, span)))
-        elif discretize == "quad":
-            end = math.sqrt(self.model.t * 0.8)
-            ddim_timesteps = (np.linspace(0, end, total_step) ** 2).astype(int)
-        else:
-            raise ValueError(f"unrecognized discretize method '{discretize}' occurred")
-        ddim_timesteps += 1
-        # calculate parameters
+        self._q_sampler = DDIMQSampler(self.model)
+        self._q_sampler.reset_buffers(discretize, total_step)
         alphas = self.model.alphas_cumprod
-        self.ddim_alphas = alphas[ddim_timesteps]
+        self.ddim_alphas = self._q_sampler.alphas
         self.ddim_alphas_prev = torch.tensor(
-            [alphas[0]] + alphas[ddim_timesteps[:-1]].tolist(),
+            [alphas[0]] + alphas[self._q_sampler.timesteps[:-1]].tolist(),
+            dtype=alphas.dtype,
             device=self.model.device,
         )
         self.ddim_sigmas = eta * torch.sqrt(
@@ -147,52 +235,34 @@ class DDIMMixin(ISampler, metaclass=ABCMeta):
             / (1.0 - self.ddim_alphas)
             * (1.0 - self.ddim_alphas / self.ddim_alphas_prev)
         )
-        self.ddim_sqrt_one_minus_alphas = torch.sqrt(1.0 - self.ddim_alphas)
-        self.ddim_timesteps = ddim_timesteps.tolist()
+        self.ddim_sqrt_one_minus_alphas = self._q_sampler.sqrt_one_minus_alphas
+        self.ddim_timesteps = self._q_sampler.timesteps.tolist()
         # unconditional conditioning
-        if unconditional_cond is None or self.model.condition_model is None:
-            self.uncond = None
-            self.uncond_guidance_scale = 0.0
-        else:
-            self.uncond = self.model._get_cond(unconditional_cond)
-            self.uncond_guidance_scale = unconditional_guidance_scale
+        self._reset_uncond_buffers(unconditional_cond, unconditional_guidance_scale)
 
 
 @ISampler.register("ddim")
 class DDIMSampler(DDIMMixin):
-    def sample_step(
+    def sample_step_core(
         self,
         image: Tensor,
         cond: Optional[Tensor],
         step: int,
         total_step: int,
+        get_eps_pred: IGetEPSPred,
+        get_denoised: IGetDenoised,
         *,
-        eta: float = 0.0,
-        discretize: str = "uniform",
-        unconditional_cond: Optional[Any] = None,
-        unconditional_guidance_scale: float = 1.0,
-        temperature: float = 1.0,
-        noise_dropout: float = 1.0,
-        quantize_denoised: bool = False,
+        eta: float,
+        discretize: str,
+        unconditional_cond: Optional[Any],
+        unconditional_guidance_scale: float,
+        temperature: float,
+        noise_dropout: float,
+        quantize_denoised: bool,
         **kwargs: Any,
     ) -> Tensor:
-        if step == 0:
-            self._reset_buffers(
-                eta,
-                discretize,
-                total_step,
-                unconditional_cond,
-                unconditional_guidance_scale,
-            )
-        self._register_temp_buffers(image, step, total_step)
-        eps_pred = self._denoise(image, self._ts, cond)
-        denoised, _ = self._get_denoised_and_pred_x0(
-            eps_pred,
-            image,
-            quantize_denoised,
-            temperature,
-            noise_dropout,
-        )
+        eps_pred = get_eps_pred(image, self._ts)
+        denoised = get_denoised(eps_pred)
         return denoised
 
 
