@@ -6,17 +6,22 @@ from typing import Any
 from typing import Dict
 from typing import Optional
 
+from .utils import append_dims
 from .protocol import ISampler
+from .protocol import IQSampler
 from .protocol import IDiffusion
+from .protocol import UncondSamplerMixin
 
 
 @ISampler.register("solver")
-class DPMSolver(ISampler):
+class DPMSolver(ISampler, UncondSamplerMixin):
     def __init__(
         self,
         model: IDiffusion,
         *,
         schedule: str = "linear",
+        unconditional_cond: Optional[Any] = None,
+        unconditional_guidance_scale: float = 1.0,
         t0: float = 1.0e-4,
         tT: Optional[float] = None,
         order: int = 3,
@@ -29,6 +34,8 @@ class DPMSolver(ISampler):
         super().__init__(model)
         if schedule not in ["linear", "cosine"]:
             raise ValueError("only `linear` & `cosine` can be used as `schedule`")
+        self.unconditional_cond = unconditional_cond
+        self.unconditional_guidance_scale = unconditional_guidance_scale
         self.beta_0 = 0.1
         self.beta_1 = 20
         self.cosine_s = 0.008
@@ -52,9 +59,15 @@ class DPMSolver(ISampler):
         self.fast_version = fast_version
         self.default_steps = default_steps
 
+    @property
+    def q_sampler(self) -> IQSampler:
+        raise NotImplementedError("`DPMSolver` has not implemented `q_sampler` yet")
+
     @property
     def sample_kwargs(self) -> Dict[str, Any]:
         return dict(
+            unconditional_cond=self.unconditional_cond,
+            unconditional_guidance_scale=self.unconditional_guidance_scale,
             t0=self.t0,
             tT=self.tT,
             order=self.order,
@@ -69,6 +82,8 @@ class DPMSolver(ISampler):
         step: int,
         total_step: int,
         *,
+        unconditional_cond: Optional[Any] = None,
+        unconditional_guidance_scale: float = 1.0,
         t0: float = 1.0e-4,
         tT: Optional[float] = None,
         order: int = 3,
@@ -79,7 +94,16 @@ class DPMSolver(ISampler):
         if tT is None:
             tT = self.default_tT
         if step == 0:
-            self._reset_buffers(total_step, t0, tT, order, skip_type, fast_version)
+            self._reset_buffers(
+                total_step,
+                unconditional_cond,
+                unconditional_guidance_scale,
+                t0,
+                tT,
+                order,
+                skip_type,
+                fast_version,
+            )
         if step >= len(self.orders):
             return image
         b = image.shape[0]
@@ -97,9 +121,9 @@ class DPMSolver(ISampler):
 
     # internal
 
-    def _denoise(self, x: Tensor, t: Tensor, cond: Optional[Tensor]) -> Tensor:
-        t = self.model.t * torch.max(t - 1.0 / self.model.t, torch.zeros_like(t))
-        return self.model.denoise(x, t, cond)
+    def _denoise(self, x: Tensor, ts: Tensor, cond: Optional[Tensor]) -> Tensor:
+        ts = self.model.t * torch.max(ts - 1.0 / self.model.t, torch.zeros_like(ts))
+        return self._uncond_denoise(x, ts, cond)
 
     def _order1_update(
         self,
@@ -118,8 +142,8 @@ class DPMSolver(ISampler):
         phi_1 = torch.expm1(h)
         noise_s = self._denoise(image, vec_s, cond)
         return (
-            torch.exp(log_alpha_t - log_alpha_s)[(...,) + (None,) * dims] * image
-            - (sigma_t * phi_1)[(...,) + (None,) * dims] * noise_s
+            append_dims(torch.exp(log_alpha_t - log_alpha_s), dims) * image
+            - append_dims(sigma_t * phi_1, dims) * noise_s
         )
 
     def _order2_update(
@@ -147,16 +171,14 @@ class DPMSolver(ISampler):
 
         noise_s = self._denoise(image, vec_s, cond)
         x_s1 = (
-            torch.exp(log_alpha_s1 - log_alpha_s)[(...,) + (None,) * dims] * image
-            - (sigma_s1 * phi_11)[(...,) + (None,) * dims] * noise_s
+            append_dims(torch.exp(log_alpha_s1 - log_alpha_s), dims) * image
+            - append_dims(sigma_s1 * phi_11, dims) * noise_s
         )
         noise_s1 = self._denoise(x_s1, s1, cond)
         return (
-            torch.exp(log_alpha_t - log_alpha_s)[(...,) + (None,) * dims] * image
-            - (sigma_t * phi_1)[(...,) + (None,) * dims] * noise_s
-            - (0.5 / r1)
-            * (sigma_t * phi_1)[(...,) + (None,) * dims]
-            * (noise_s1 - noise_s)
+            append_dims(torch.exp(log_alpha_t - log_alpha_s), dims) * image
+            - append_dims(sigma_t * phi_1, dims) * noise_s
+            - (0.5 / r1) * append_dims(sigma_t * phi_1, dims) * (noise_s1 - noise_s)
         )
 
     def _order3_update(
@@ -192,30 +214,27 @@ class DPMSolver(ISampler):
 
         noise_s = self._denoise(image, vec_s, cond)
         x_s1 = (
-            torch.exp(log_alpha_s1 - log_alpha_s)[(...,) + (None,) * dims] * image
-            - (sigma_s1 * phi_11)[(...,) + (None,) * dims] * noise_s
+            append_dims(torch.exp(log_alpha_s1 - log_alpha_s), dims) * image
+            - append_dims(sigma_s1 * phi_11, dims) * noise_s
         )
         noise_s1 = self._denoise(x_s1, s1, cond)
         x_s2 = (
-            torch.exp(log_alpha_s2 - log_alpha_s)[(...,) + (None,) * dims] * image
-            - (sigma_s2 * phi_12)[(...,) + (None,) * dims] * noise_s
-            - r2
-            / r1
-            * (sigma_s2 * phi_22)[(...,) + (None,) * dims]
-            * (noise_s1 - noise_s)
+            append_dims(torch.exp(log_alpha_s2 - log_alpha_s), dims) * image
+            - append_dims(sigma_s2 * phi_12, dims) * noise_s
+            - r2 / r1 * append_dims(sigma_s2 * phi_22, dims) * (noise_s1 - noise_s)
         )
         noise_s2 = self._denoise(x_s2, s2, cond)
         return (
-            torch.exp(log_alpha_t - log_alpha_s)[(...,) + (None,) * dims] * image
-            - (sigma_t * phi_1)[(...,) + (None,) * dims] * noise_s
-            - (1.0 / r2)
-            * (sigma_t * phi_2)[(...,) + (None,) * dims]
-            * (noise_s2 - noise_s)
+            append_dims(torch.exp(log_alpha_t - log_alpha_s), dims) * image
+            - append_dims(sigma_t * phi_1, dims) * noise_s
+            - (1.0 / r2) * append_dims(sigma_t * phi_2, dims) * (noise_s2 - noise_s)
         )
 
     def _reset_buffers(
         self,
         total_step: int,
+        unconditional_cond: Optional[Any],
+        unconditional_guidance_scale: float,
         t0: float,
         tT: float,
         order: int,
@@ -237,6 +256,8 @@ class DPMSolver(ISampler):
             N_steps = total_step // order
             self.orders = [order] * N_steps
             self.timesteps = self._get_time_steps(skip_type, t0, tT, N_steps)
+        # unconditional conditioning
+        self._reset_uncond_buffers(unconditional_cond, unconditional_guidance_scale)
 
     def _get_time_steps(self, skip_type: str, t0: float, tT: float, N: int) -> Tensor:
         device = self.model.device
